/**
	PeerServer handles the task of connecting new PeerTheaters to the
	IO network.  A PeerTheater connects to the PeerServer, and is added
	to the server's list of peers.  The server then sends back to the
	Peer theater a list of peers on the network that it can connect to.

        Author: Travis Desell
*/

package io.core.server;

import java.io.*;
import java.net.*;
import java.util.Vector;

public class PeerServer {
	public static Vector peers = new Vector();
	public static int maxTheaters = 100;	

	private static ServerSocket passive;
	private static int listenPort = 5050;


	private PeerServer() {
		peers = new Vector();
	}

	public static void listen(int port) {
		try {
			passive = new ServerSocket(port);
		} catch (IOException e) {
			System.err.println( "IOException creating server socket." + e.getMessage() );

			return;
		}

		listenPort = passive.getLocalPort();
		System.out.println("PeerServer listening on port: " + listenPort);

		//Loop forever, accepting incoming connections
		while (true) {
			Socket incoming = null;

			//accept the connection
			try {
				incoming = passive.accept();
			} catch (IOException e) {
				System.err.println( "IOException occured accepting new connection" + e.getMessage() );

				return;
			}

			PeerHandler peerHandler = new PeerHandler(incoming, peers, maxTheaters);
			peerHandler.start();
		}
	}

	public static void main(String[] args) {
		if (args.length > 0) {
			if (args.length == 1) {
				if ((args[0].equals("-h")) || (args[0].equals("-?"))) {
					PeerServer.helpMessage();
				} else if (args[0].equals("-v")) {
					PeerServer.printServerVersion();
				} else {
					PeerServer.helpMessage("illegal option");
				}
			} else if (args.length != 4) {
				PeerServer.helpMessage("wrong arguments");
			} else if ( (!args[0].equals("-p") || !args[2].equals("-m")) && 
				    (!args[0].equals("-m") || !args[2].equals("-p")) ) {
				PeerServer.helpMessage("wrong options");
			}  else {
				if (args[0].equals("-m")) {
					try {
						maxTheaters = Integer.parseInt(args[1]);
					} catch (Exception e) {
						PeerServer.helpMessage("invalid port number : " + args[1]);
					}
								
					try {
						listenPort = Integer.parseInt(args[3]);
					} catch (Exception e) {
						PeerServer.helpMessage("invalid port number : " + args[1]);
					}
				} else {
					try {
						maxTheaters = Integer.parseInt(args[3]);
					} catch (Exception e) {
						PeerServer.helpMessage("invalid port number : " + args[1]);
					}
								
					try {
						listenPort = Integer.parseInt(args[1]);
					} catch (Exception e) {
						PeerServer.helpMessage("invalid port number : " + args[1]);
					}
				}
			}
		}

		PeerServer.listen( listenPort );
	}

	private static void printServerVersion() {
		System.out.println("IO Peer Server Version: 0.1");
		System.exit(0);
	}

	private static void helpMessage(String s) {
		System.out.println(s);
		PeerServer.printHelpMessage();
		System.exit(1);
	}

	private static void helpMessage() {
		PeerServer.printHelpMessage();
		System.exit(0);
	}

	private static void printHelpMessage() {
		System.out.println("usage :");
		System.out.println("  java ...PeerServer");
		System.out.println("  java ...PeerServer -h");
		System.out.println("  java ...PeerServer -v");
		System.out.println("  java ...PeerServer -p portNumber");
		System.out.println("                     -m maxTheaters");
		System.out.println("options :");
		System.out.println("  -h : print this message");
		System.out.println("  -v : print version");
		System.out.println("  -p portNumber   : set the listening port to portNumber");
		System.out.println("                    default port number is 5050");
		System.out.println("  -m maxTheaters  : maxTheaters is the maximum number of ");
		System.out.println("                    theaters the Peer Server will keep in");
		System.out.println("                    its theater list.");
	}
}
