package salsa.messaging;

import salsa.language.ActorState;
import salsa.language.Message;
import salsa.naming.UAN;
import salsa.naming.UAL;

/**
	This Interface represents the most basic services a theater must provide a salsa program.
	
	@author stepha
*/
public interface TheaterService {
	/**
		This method returns an actorstate, given a specific UAN.
		If the actorstate for a given UAN is not local to the theater,
		it will return the UAL of that actor
	*/
	public Object getEntry(UAN uan);

	/**
		This method adds an actorstate to the theaters registry of 
		states.
	*/
	public void setEntry(UAN uan, Object entry);

	/**
		This is used for on-the-fly theater creation,
		it creates a UAL for an anonymous actor.
	*/
	public UAL generateUAL();


	/**
		This method returns an object implementing TheaterService. 
		Used for on-the-fly theater startup. (The theater instance might exist but it may only provide
		passive serivces and not listen for incoming actors or messages.) The method will update the 
		actor's UAL after theater startup is successful. If this method is called at the theater already exists,
		the inital actor is registered with the existing theater if it is specified.
		
		@param port The port which the theater should start listening on. 0 indicates any port.
		@param initial Reference to the actor requesting theater startup. 
	*/
	public TheaterService startTheater( int port, ActorState initial );
 

	/**
		This method returns the port that the theater is currently listening on i.e. 4040.
		If the theater is not actively listening, -1 is returned.
	*/
	public int getPort();
}
