/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */
package wwc.messaging;

import java.net.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Vector;
import salsa.language.*;
import salsa.messaging.TheaterService;
import salsa.language.ServiceFactory;

public class BasicHandler {
	private TheaterService theater;
	
	public BasicHandler(TheaterService _theater) {
		theater = _theater;
	}

	public void processActor(ActorState incomingState, Socket incoming) {
		//add the state to the stateRegistry, so incoming actor references can
		//get to it
		theater.setEntry(incomingState.getUAN(), incomingState);

		RunTime.receivedUniversalActor();

		// Send an RmspAcknowledgement.
		try {
			new ObjectOutputStream( incoming.getOutputStream() ).writeObject( new RMSPAcknowledgement() );
		} catch (IOException e) {
			System.err.println( "Could not send RmspAcknowledgement is the remote theater compliant? " + e.getMessage() );
		}
	}

	public void processMessage(Message m) {
		// Send the message.
		m.getTarget().send(m);
	}

	public void process(Object recieved, Socket incoming) {
		if (recieved instanceof ActorState) {
			ActorState incomingActor = (ActorState) recieved;
			processActor(incomingActor, incoming);
		} else if (recieved instanceof Message) {  // just serializing data
	        	Message m = (Message) recieved;
			processMessage(m);
      		}
   	}
}

