package wwc.messaging;

import java.io.ObjectOutputStream;
import java.io.IOException;
import java.net.Socket;


import salsa.language.*;
import salsa.language.exceptions.*;
import salsa.naming.*;
import salsa.messaging.*;

/**
 * This class is the reference implementation of the MessagingService. The
 * service is implemented as a RMSP client.
 * 
 * This implementation uses the NamingService interface and the salsa.language.
 * ServiceFactory for UAN resolution.
 * 
 * @author stepha
 */
public class Messenger implements MessagingService {

	/**
	 * This is the maximum number of times a message maybe forwarded before
	 * the name server is queried. Set by system property -DforwardThreshold=<n>
	 * The default value is 4.
	 */
	private int forwarderThreshold;


	/**
	 * Constructor for Messenger.
	 */
	public Messenger() {
		super();
		
		String thresh = System.getProperty( "forwardThreshold" );
		if (thresh != null) {
			forwarderThreshold = Integer.parseInt( thresh );
		}
		else {
			forwarderThreshold = 4;
		}
	}


	/**
	 * This method sends the specified message using RMSP.
	 * @author stepha
	 * @version $Id: Messenger.java,v 1.1 2003/03/25 07:42:07 deselt Exp $
	 * @see salsa.language.MessagingService#send(salsa.language.Message)
	 */
	public void send(Message message, UAL target) {
		// Determine the destination of this message.
		Socket outgoing = null;
		
		if ((message.getForwarded() >= forwarderThreshold) || (target == null)) {
			UAN uan = message.getTarget().getUAN();

			// Resolve the location.
			target = ServiceFactory.getNaming().resolve( uan );
			message.setForwarded( 0 );

			ServiceFactory.getTheater().setEntry(uan, target);
		}

		try {
			// Attempt to connect to the target.
			outgoing = new Socket( target.getHost(), target.getPort() );
		} catch (IOException e) {
			throw new SalsaWrapperException( "Can't send message, invalid location.", e );
		}


		// Update the number of times the message has been forwarded.
		message.setForwarded( message.getForwarded() + 1 );
		
		// Send the message.
		try {
			ObjectOutputStream outStream = new ObjectOutputStream( outgoing.getOutputStream() );
			outStream.writeObject( message );
			outStream.close();
			outgoing.close();
		} catch (IOException e) {
			throw new SalsaWrapperException( "Error sending message: ", e );
		}
	}
}
