package wwc.messaging;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.net.Socket;

import salsa.language.*;
import salsa.language.exceptions.*;
import salsa.messaging.*;
import salsa.naming.*;

/**
 * This class implements an RMSP Client.
 * 
 * @author Abe Stephens -- stepha@rpi.edu
 */
public class Migrator implements MigrationService {

	/**
	 * Constructor for Migrator.
	 */
	public Migrator() {
		super();
	}

	/**
	 * This method attempts to migrate the specified actor to the specified
	 * location using RMSP.
	 * 
	 * @see salsa. language. MigrationService#migrate (salsa. language.
	 * ActorState, salsa. language.UAL)
	 */
	public void migrate(ActorState state, UAL target) {
		ServiceFactory.getTheater().setEntry(state.getUAN(), target);

		Socket outgoing = null;

		if (target != null) {
			try {
				// Attempt to connect to the target.
				outgoing = new Socket(target.getHost(), target.getPort());
			} catch (IOException e) {
				throw new SalsaWrapperException("Can't send message, invalid location.",e);
			}
		} else {
			// Attempting to send a message without a UAN or UAL.
			throw new SalsaException("Can't send message, target must have UAN or UAL.");
		}

		// Send the message.
		ObjectOutputStream outStream;
		try {
			outStream = new ObjectOutputStream(outgoing.getOutputStream());
			outStream.writeObject( state );
		} catch (IOException e) {
			throw new SalsaWrapperException("Error sending message: ", e);
		}
		
		// Wait for the Acknowledgement that the actor is situated on the remote theater.
		try {
			RMSPAcknowledgement ack = (RMSPAcknowledgement) new ObjectInputStream( outgoing.getInputStream() ).readObject();
			outStream.close();
			outgoing.close();
		} catch (IOException e) {
			throw new SalsaWrapperException( "Did not recieve RmspAcknowledgement.", e );
		} catch (ClassNotFoundException e) {
			throw new SalsaWrapperException( "Did not recieve RmspAcknowledgement.", e );
		}
	}
}
